/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "desktop_watcher.h"
#include <QDir>
#include <QEventLoop>

DesktopWatcher::DesktopWatcher()
{
    //监控.desktop文件目录
    m_fileWatcher = new QFileSystemWatcher;
    m_fileWatcher->addPaths(QStringList() << QString("/usr/share/applications")
                            << QString(QDir::homePath() + "/.local/share/applications/")
                            << QString("/var/lib/snapd/desktop/applications/")
                            << QString("/var/lib/flatpak/exports/share/applications/"));
    connect(m_fileWatcher, &QFileSystemWatcher::directoryChanged, this, &DesktopWatcher::directoryChangedSlot);
    //    m_recentFile = new QFileSystemWatcher;
    //    m_recentFile->addPath(QString(QDir::homePath()+"/.local/share/recently-used.xbel"));
    //    connect(m_recentFile,&QFileSystemWatcher::fileChanged,this,[=](){
    //        m_recentFile->addPath(QDir::homePath()+"/.local/share/recently-used.xbel");
    //    Q_EMIT updateRecentList();});
    m_directoryChangedThread = new DirectoryChangedThread;
    connect(m_directoryChangedThread, &DirectoryChangedThread::requestUpdateSignal, this, &DesktopWatcher::requestUpdateSlot);
}

DesktopWatcher::~DesktopWatcher()
{
}

/**
 * desktop文件目录改变信号槽
 */
void DesktopWatcher::directoryChangedSlot(const QString &path)
{
    QEventLoop loop;
    QTimer::singleShot(100, &loop, SLOT(quit()));
    loop.exec();
    m_directoryChangedThread->start();
}

void DesktopWatcher::requestUpdateSlot()
{
    m_directoryChangedThread->quit();
    Q_EMIT directoryChangedSignal();
}
