#include "push_button.h"

PushButton::PushButton(QWidget *parent) : QPushButton(parent)
{
    m_btnLayout = new QHBoxLayout(this);
    this->setLayout(m_btnLayout);
    m_label = new IconLabel(this);
    m_btnLayout->addWidget(m_label);

    m_label->setAlignment(Qt::AlignCenter);
    m_btnLayout->setAlignment(Qt::AlignCenter);
}

void PushButton::setLabel(QPixmap pixmap, int size)
{
    m_label->setFixedSize(size, size);
    m_label->setIcon(pixmap);
}

PushButton::~PushButton()
{

}

void PushButton::paintEvent(QPaintEvent *e)
{
    Q_UNUSED(e);
    QStylePainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    QStyleOptionButton option;
    initStyleOption(&option);

    if (option.state & QStyle::State_Sunken) {
        painter.save();
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::white);
        if (g_curStyle == "ukui-dark" || g_isFullScreen) {
            painter.setOpacity(0.25);
        } else {
            painter.setOpacity(0.65);
        }

        painter.drawRoundedRect(option.rect.adjusted(1, 1, -1, -1), 6, 6);
        painter.restore();
    }

    if (option.state & QStyle::State_MouseOver) {
        painter.save();
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::white);
        if (g_curStyle == "ukui-dark" || g_isFullScreen) {
            painter.setOpacity(0.1);
        } else {
            painter.setOpacity(0.35);
        }

        painter.drawRoundedRect(option.rect.adjusted(1, 1, -1, -1), 6, 6);
        painter.restore();
    }

    if (option.state & QStyle::State_On) {
        painter.save();
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::white);
        if (g_curStyle == "ukui-dark" || g_isFullScreen) {
            painter.setOpacity(0.2);
        } else {
            painter.setOpacity(0.5);
        }

        painter.drawRoundedRect(option.rect.adjusted(1, 1, -1, -1), 6, 6);
        painter.restore();
    }

    if (option.state & QStyle::State_HasFocus) {
        painter.save();
        QStyleOption opt;
        QColor color = opt.palette.color(QPalette::Highlight);
        painter.setPen(QPen(color, 2));
        painter.drawRoundedRect(option.rect.adjusted(1, 1, -1, -1), 6, 6);
        painter.restore();
    }
}

IconLabel::IconLabel(QWidget *parent)
    : QLabel(parent)
{
    this->setContentsMargins(0, 0, 0, 0);
}

void IconLabel::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::SmoothPixmapTransform, true);
    painter.drawPixmap(0, 0, m_pixmap);
    return QLabel::paintEvent(event);
}

void IconLabel::setIcon(const QPixmap &pixmap)
{
    m_pixmap = pixmap;
}


