/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <QPainter>
#include "klistview.h"
#include "utility.h"
#include <QScrollBar>
#include <QtConcurrent/QtConcurrent>
#include "rightclickmenu.h"

KListView::KListView(QWidget *parent):
    QListView(parent)
{
}

KListView::~KListView()
{
    if (m_delegate) {
        delete m_delegate;
        m_delegate = nullptr;
    }

    if (m_listmodel) {
        delete m_listmodel;
        m_listmodel = nullptr;
    }
}

void KListView::addData(QStringList data)
{
    m_listmodel = new QStandardItemModel(this);
    this->setModel(m_listmodel);

    Q_FOREACH(QString desktopfp, data) {
        QStandardItem *item = new QStandardItem;
        item->setData(QVariant::fromValue<QString>(desktopfp), Qt::DisplayRole);
        m_listmodel->appendRow(item);
    }
}

void KListView::updateData(QStringList data)
{
    m_listmodel->clear();

    Q_FOREACH(QString desktopfp, data) {
        QStandardItem *item = new QStandardItem;
        item->setData(QVariant::fromValue<QString>(desktopfp), Qt::DisplayRole);
        m_listmodel->appendRow(item);
    }
}

void KListView::onClicked(QModelIndex index)
{
    QVariant var = m_listmodel->data(index, Qt::DisplayRole);
    if (var.isValid()) {
        QString desktopfp = var.value<QString>();
        Q_EMIT sendHideMainWindowSignal();
        QtConcurrent::run(execApp, QString(desktopfp));
    }
}

void KListView::paintEvent(QPaintEvent *e)
{
    double transparency = getTransparency();
    QPainter painter(this->viewport());
    painter.setBrush(this->palette().base());
    painter.setPen(Qt::transparent);
    painter.setOpacity(transparency);
    painter.drawRect(this->rect());
    QListView::paintEvent(e);
}

void KListView::mouseMoveEvent(QMouseEvent *e)
{
    this->clearFocus();
    return QListView::mouseMoveEvent(e);
}

void KListView::mousePressEvent(QMouseEvent *event)
{
    m_pressPos = event->pos();
    if ((this->indexAt(m_pressPos).isValid()) && event->button() == Qt::LeftButton) {
        m_pressApp = m_listmodel->data(this->indexAt(m_pressPos), Qt::DisplayRole);
    }
    return QListView::mousePressEvent(event);
}

void KListView::mouseReleaseEvent(QMouseEvent *e)
{
    if (!(this->indexAt(e->pos()).isValid()) && e->button() == Qt::LeftButton) {
        Q_EMIT sendHideMainWindowSignal();
    } else {
        return QListView::mouseReleaseEvent(e);
    }
}

void KListView::rightClickedSlot(const QPoint &pos)
{
    if (!(this->selectionModel()->selectedIndexes().isEmpty())) {

        QModelIndex index = indexAt(pos);
        if(!index.isValid()) {
             return;
        }

        QVariant var = m_listmodel->data(index, Qt::DisplayRole);
        QStringList strlist = var.value<QStringList>();

        if (strlist.count() == 1 || ((strlist.count() == 2) && (strlist.at(1).toInt() == 1))) {
            RightClickMenu menu;
            int ret = menu.showAppBtnMenu(this->mapToGlobal(pos), strlist.at(0));

            switch (ret) {
                case 1:
                case 2:
                    Q_EMIT sendUpdateAppListSignal();
                    break;

                case 6:
                case 7:
                    Q_EMIT sendHideMainWindowSignal();
                    break;

                case 18:
                    Q_EMIT sendCollectViewUpdate();
                    break;

                default:
                    break;
            }
        }
    }
}
