/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef KLISTVIEW_H
#define KLISTVIEW_H
#include <QListView>
#include "kitemdelegate.h"
#include <QStandardItemModel>
#include <QEvent>
#include <QMouseEvent>

class KListView : public QListView
{
    Q_OBJECT
public:
    KListView(QWidget *parent);
    ~KListView();
    void initWidget();

    KItemDelegate *m_delegate = nullptr;
    QStandardItemModel *m_listmodel = nullptr;
    int module = 0;
    double m_transparency;
    QVariant m_pressApp;
    QPoint m_pressPos;
    UkuiMenuInterface *m_ukuiMenuInterface = nullptr;
protected:
    void paintEvent(QPaintEvent *e);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent *e);
public:
    void addData(QStringList data);
    void updateData(QStringList data);

Q_SIGNALS:
    void sendHideMainWindowSignal();
    void sendItemClickedSignal(QString arg);//发送item点击信号
    void sendUpdateAppListSignal();
    void sendCollectViewUpdate();

public Q_SLOTS:
    void rightClickedSlot(const QPoint &pos);
    void onClicked(QModelIndex index);
};

#endif // KLISTVIEW_H
