/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "rotationlabel.h"
#include <QPainter>
#include <QDebug>
#include "utility.h"

RotationLabel::RotationLabel(QWidget *parent)
    : QLabel(parent)
{
    this->setFrameShape(QFrame::NoFrame);
    this->setContentsMargins(0, 0, 0, 0);
}

void RotationLabel::setRotation(int rot)
{
    m_rotation = rot;
    this->update();
    qDebug() << m_rotation;
}

int RotationLabel::rotation()
{
    return m_rotation;
}

void RotationLabel::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::SmoothPixmapTransform, true);
    painter.translate(rect().width() / 2, rect().height() / 2);
    painter.rotate(m_rotation);
    painter.translate(-rect().width() / 2, -rect().height() / 2);
    painter.drawPixmap(rect().adjusted(0, 9, 0, -9), m_pixmap);
    return QLabel::paintEvent(event);
}

void RotationLabel::mouseReleaseEvent(QMouseEvent *ev)
{
    Q_UNUSED(ev);
}

void RotationLabel::setIcon(const QPixmap &pixmap)
{
    m_pixmap = pixmap;
}
