/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef RIGHTCLICKMENU_H
#define RIGHTCLICKMENU_H

#include <QObject>
#include <QWidget>
#include <QHBoxLayout>
#include <QMenu>
#include <QAction>
#include <QSvgRenderer>
#include <QPainter>
#include <QWidgetAction>
#include <QLabel>
#include <QSettings>
#include <QDir>
#include <QFileInfo>
#include <QDBusInterface>
#include <QProcess>
#include <QEvent>
#include "ukuimenuinterface.h"
#include "style.h"
#include "menubox.h"

#define DBUS_SESSION_NAME       "org.gnome.SessionManager"
#define DBUS_SESSION_PATH       "/org/gnome/SessionManager"
#define DBUS_SESSION_INTERFACE  "org.gnome.SessionManager"

class RightClickMenu : public QWidget
{
    Q_OBJECT
public:
    /**
     * @brief Right-click menu
     * @param module: Right-click menu type, Only the following parameters can be entered:
     *  0: application
     *  1: power
     *  2: others
     */
    explicit RightClickMenu(QWidget *parent = nullptr);
    virtual ~RightClickMenu();
    /**
     * @brief Displays the application right-click menu
     * @param desktopfp: Application desktop file
     * @return return action number
     */
    int showAppBtnMenu(const QPoint &pos, QString desktopfp);
    /**
     * @brief Displays the power right-click menu
     * @return return action number
     */
    int showShutdownMenu(const QPoint &pos, const bool &isFullWind);
    /**
     * @brief Displays other application right-click menus
     * @param desktopfp: Application desktop file
     * @return return action number
     */
    int showOtherMenu(const QPoint &pos, QString desktopfp);

private:
    QString m_desktopfp;
    int m_actionNumber;
    QProcess *m_cmdProc = nullptr;
    UkuiMenuInterface *m_ukuiMenuInterface = nullptr;
    QStringList m_whiteList;

protected:
    /**
     * @brief Get icon pixmap
     * @param iconstr: Icon names
     * @param type: Icon type, Only the following parameters can be entered:
     *  0: Resource icon
     *  1: System icon
     * @return
     */
    QPixmap getIconPixmap(QString iconstr, int type);

    bool hasMultipleUsers();

    bool canSwitch();

private Q_SLOTS:
    /**
     * @brief Fixed to all software
     */
    void fixToAllActionTriggerSlot();
    /**
     * @brief Unfix from all software
     */
    void unfixedFromAllActionTriggerSlot();
    /**
     * @brief Fixed to the taskbar
     */
    void fixToTaskbarActionTriggerSlot();
    /**
     * @brief Unfix from the taskbar
     */
    void unfixedFromTaskbarActionTriggerSlot();
    /**
     * @brief Add to the desktop shortcut
     */
    void addToDesktopActionTriggerSlot();
    /**
     * @brief Uninstall
     */
    void uninstallActionTriggerSlot();
    /**
     * @brief Read command output
     */
    void onReadOutput();
    /**
     * @brief Attribute
     */
    void attributeActionTriggerSlot();
    /**
     * @brief Lock Screen
     */
    void lockScreenActionTriggerSlot();
    /**
     * @brief Switch user
     */
    void switchUserActionTriggerSlot();
    /**
     * @brief Log out
     */
    void logoutActionTriggerSlot();
    /**
     * @brief Reboot
     */
    void rebootActionTriggerSlot();
    /**
     * @brief Shut down
     */
    void shutdownActionTriggerSlot();
    void hibernateActionTriggerSlot();
    void sleepActionTriggerSlot();
    /**
     * @brief Personalize this list
     */
    void otherListActionTriggerSlot();
    void pincToCollectionActionTriggerSlot();
    void removeFromCollectionActionTriggerSlot();

public:
Q_SIGNALS:
    void sendMainWinActiveSignal(bool flag);
};

#endif // RIGHTCLICKMENU_H
