/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FUNCTIONWIDGET_H
#define FUNCTIONWIDGET_H

#include <QWidget>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QLocale>
#include <QDateTime>
#include <QGSettings>
#include <QSettings>
#include <QGraphicsDropShadowEffect>
#include "plugin_widget.h"
#include "currenttime_interface.h"

class FunctionWidget : public QWidget
{
    Q_OBJECT
public:
    FunctionWidget(QWidget *parent);
    ~FunctionWidget();
    //QWidget* focusPlug=nullptr;
    PluginWidget *focusPlug = nullptr;
    void setDownOpacityEffect(const qreal &num);

protected:
    void initUi();

private:
    CurrentTimeInterface *Time = nullptr;
//    QGSettings *themeSetting = nullptr;
    QGSettings *timeSetting = nullptr;
    QString themeName;
    QObject *plugin = nullptr;
    QGraphicsDropShadowEffect *effect;
    QPalette palette;
    QTimer *myTimer = nullptr;

    //上方(时间+搜索)
    QWidget *upWidget = nullptr;
    QWidget *leftUpWidget = nullptr;
    QWidget *upLeftWidget = nullptr;
    QWidget *downWidget = nullptr;


    QVBoxLayout *upLayout;
    QHBoxLayout *leftUpLayout;
    QVBoxLayout *downLayout;
    QVBoxLayout *focusModeLayout;

    QHBoxLayout *upLeftLayout;
    QVBoxLayout *upRightLayout;

    QLabel *timeLabel;
    QLabel *weekLabel;
    QLabel *dateLabel;
    QPushButton *searchEditBtn;

    QDBusInterface *usrInterface = nullptr;

    //右侧
    QWidget *upRightWidget = nullptr;

    qreal opacity; //透明系数

private Q_SLOTS:

    void obtainSearchResult();

public Q_SLOTS:

    void changeSearchBoxBackground(QString styleName);
public:
Q_SIGNALS:
    void hideTabletWindow();



};

#endif // TIMEWIDGET_H
