/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "main_view_widget.h"
#include "style.h"
#include <QSvgRenderer>
#include <QPainter>
#include <syslog.h>
#include <QDebug>
#include <thread>
#include <KWindowInfo>
#include <KWindowSystem>
#include <KWindowEffects>
#include <QCheckBox>

MainViewWidget::MainViewWidget(QWidget *parent) :
    QWidget(parent)
{
    this->setWindowFlags(Qt::CustomizeWindowHint | Qt::FramelessWindowHint);
    this->setAttribute(Qt::WA_StyledBackground, true);
    this->setAutoFillBackground(false);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setFocusPolicy(Qt::StrongFocus);
}

MainViewWidget::~MainViewWidget()
{
}

void MainViewWidget::styleChangeSlot(const QString &style)
{
    m_curStyle = style;

    if (style == "ukui-dark") {
        m_backColor.setNamedColor("#800D0D0D");
    } else {
        m_backColor.setNamedColor("#94FFFFFF");
    }
}

void MainViewWidget::paintEvent(QPaintEvent *event)
{
    QRect rect = this->rect().adjusted(5, 5, 0, -5);
    QPainterPath path;
    QPainter painter(this);
    path.addRoundedRect(rect, 10, 10);
    painter.setRenderHint(QPainter::Antialiasing);  // 反锯齿;
    painter.setPen(Qt::transparent);
    painter.setOpacity(0.58);
    painter.fillPath(path, m_backColor);
    QWidget::paintEvent(event);
}
